/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#pragma once
#include <ders/config.hpp>
#include <vector>
#include <ders/exception.hpp>

namespace ders {

inline bool is_sepr(char ch) { return ch=='/' || ch=='\\'; }
inline bool is_delr(char ch) { return is_sepr(ch) || ch==':'; }

char last_sepr(strsz path);
strsz last_name(strsz path);
un_text path_add(mem_pool* mp, strsz path, strsz name);

std::vector<sh_text> list_dir(mem_pool* mp, strsz path, std::error_code* ec);
std::vector<sh_text> ex_list_dir(mem_pool* mp, strsz path);

enum fi_type {
    ft_file =1,
    ft_dir  =2,
    ft_other=3
};

struct file_info {
    long long size;
    fi_type type;

    file_info() : size(0), type(ft_other) {}
    file_info(long long s, fi_type t) : size(s), type(t) {}
};

file_info get_info(mem_pool* mp, strsz path, std::error_code* ec);
file_info ex_get_info(mem_pool* mp, strsz path);

}  // namespace ders
