/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#pragma once
#include <ders/config.hpp>
#include <unordered_map>
#include <unordered_set>
#include <ders/mp_allocator.hpp>
#include <ders/text.hpp>

namespace ders {

size_t mem_hash(const void* ptr, int sz);

template<class Key, class T, class Hash=std::hash<Key>, class KeyEqual=std::equal_to<Key>> 
using mp_unordered_map=std::unordered_map<Key, T, Hash, KeyEqual, mp_allocator<std::pair<const Key,T>>>;

template<class Key, class Hash=std::hash<Key>, class KeyEqual=std::equal_to<Key>> 
using mp_unordered_set=std::unordered_set<Key, Hash, KeyEqual, mp_allocator<Key>>;

}  // namespace ders

namespace std {

template<>
struct hash<ders::sh_text> {
    size_t operator()(const ders::sh_text& st) const noexcept 
    { 
        return ders::mem_hash(st->begin(), st->size()); 
    }
};

}  // namespace std
