/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#pragma once
#include <ders/config.hpp>
#include <stddef.h>

namespace ders {

namespace impl {

struct mem_pool_data;

}  // namespace impl

class mem_pool {
 public:
    mem_pool();
    ~mem_pool();

    void* alloc(size_t sz);
    void free(void* ptr, size_t sz);

    mem_pool(const mem_pool&)=delete;
    mem_pool& operator=(const mem_pool&)=delete;

 private:    
    impl::mem_pool_data* data;
};

class mp_buf {
 public:
    mp_buf(mem_pool* mp, int sz) : mp_(mp), sz_(sz), ptr_(mp->alloc(sz)) {}
    ~mp_buf() { if (ptr_) mp_->free(ptr_, sz_); }

    mem_pool* pool() const { return mp_; }
    void* buf() const { return ptr_; }
    int size() const { return sz_; }

    void* release() 
    {
        void* ret=ptr_;
        ptr_=0;
        return ret;
    }

    mp_buf(const mp_buf&)=delete;
    mp_buf& operator=(const mp_buf&)=delete;

 private:    
    mem_pool* mp_;
    int sz_;
    void* ptr_;
};

}  // namespace ders
