/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#pragma once
#include <ders/config.hpp>
#include <stdint.h>
#include <ders/text.hpp>

namespace ders {

namespace impl {

class wrap_ptr {
 public:
    wrap_ptr(const char* pc) : i_((intptr_t)pc) {}
    wrap_ptr(const void* pv) : i_(-(intptr_t)pv) {}

    bool is_str() const { return i_>0; }

    const char* str() const { return (const char*)i_; }
    const void* ptr() const { return (const void*)-i_; }

 private:
    intptr_t i_;
};

}  // namespace impl

class tx_buf {
 public:
    explicit tx_buf(mem_pool* mp, int rsrv=1024) : ut_(make_un<text>(mp, mp, rsrv)) {}

    tx_buf(const tx_buf&)=delete;
    tx_buf& operator=(const tx_buf&)=delete;

    template<size_t N> tx_buf& operator+(const char (&chn)[N]) { *ut_+=strsz(chn, N-1); return *this; }

    tx_buf& operator+(impl::wrap_ptr wp)
    {
        if (wp.is_str()) *ut_+=strsz(wp.str(), strlen(wp.str()));
        else append(wp.ptr());

        return *this;
    }

    tx_buf& operator+(strsz s) { *ut_+=s; return *this; }
    tx_buf& operator+(char ch) { *ut_+=ch; return *this; }

    tx_buf& operator+(const text& t) { *ut_+=t.str(); return *this; }
    tx_buf& operator+(const un_text& ut) { *ut_+=ut->str(); return *this; }
    tx_buf& operator+(const sh_text& st) { *ut_+=st->str(); return *this; }

    tx_buf& operator+(int v);
    tx_buf& operator+(unsigned int v);
    tx_buf& operator+(long v);
    tx_buf& operator+(unsigned long v);
    tx_buf& operator+(long long v);
    tx_buf& operator+(unsigned long long v);

    operator strsz() const { return ut_->str(); }
    un_text release() { return std::move(ut_); }

 private:
    un_text ut_;

    void append(const void* ptr);
};

}  // namespace ders
