/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#include <ders/dir.hpp>

namespace ders {

using namespace std;    

char last_sepr(strsz path)
{
    for (int i=path.size()-1; i>=0; i--) {
        char ch=path[i];
        if (is_sepr(ch)) return ch;
    }

    return '/';
}

strsz last_name(strsz path)
{
    auto beg=path.end();
    for (int i=path.size()-1; i>=0 && !is_delr(path[i]); i--) beg=path.begin()+i;

    return strsz(beg, path.end()-beg);
}

un_text path_add(mem_pool* mp, strsz path, strsz name)
{
    tx_buf tb(mp);
    tb+path;

    if (name.size()) {
        if (path.size() && !is_delr(path.back())) tb+last_sepr(path);
        tb+name;
    }

    return tb.release();
}

vector<sh_text> ex_list_dir(mem_pool* mp, strsz path)
{
    error_code ec;

    auto ret=list_dir(mp, path, &ec);
    if (ec) throw newExc<FileException>(mp, mp, FLLN, "Can't list dir", ec, path);

    return ret;
}

file_info ex_get_info(mem_pool* mp, strsz path)
{
    error_code ec;

    file_info ret=get_info(mp, path, &ec);
    if (ec) throw newExc<FileException>(mp, mp, FLLN, "Can't get info", ec, path);

    return ret;
}

}  // namespace ders
