/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#include <ders/exception.hpp>

namespace ders {

using namespace std;    

un_text Exception::toText() const
{
    return (tx_buf(pool())+className()+" ["+loc_.file+":"+loc_.line+"], message: "+msg_).release();
}

un_text StdException::toText() const
{
    return (tx_buf(pool())+className()+" ["+loc_.file+":"+loc_.line+"], type=\""+ti_->name()+"\", message: "+
        msg_).release();
}

un_text ErrorException::toText() const
{
    return (tx_buf(pool())+className()+" ["+loc_.file+":"+loc_.line+"], error=("+ec_.value()+", \""+
        ec_.message().c_str()+"\", \""+ec_.category().name()+"\"), message: "+msg_).release();
}

un_text FileException::toText() const
{
    return (tx_buf(pool())+className()+" ["+loc_.file+":"+loc_.line+"], path=\""+path_+"\", error=("+
        ec_.value()+", \""+ec_.message().c_str()+"\", \""+ec_.category().name()+"\"), message: "+msg_).release();
}

un_text ExitMsgException::toText() const
{
    return (tx_buf(pool())+className()+" ["+loc_.file+":"+loc_.line+"], exitCode="+ecd_+", message: "+msg_).release();
}

un_text toTextAll(const Exception* exc)
{
    tx_buf tb(exc->pool());    

    for (auto e=exc; e; ) {
        tb+e->toText();
        
        e=get(e->nested());
        if (!e) break;

        tb+'\n';
    }

    return tb.release();
}

sh_ptr<Exception> recatch(mem_pool* mp, FileLine loc, bool rethrowMsg)
{
    try { throw; }
    catch (sh_ptr<Exception> se) {
        if (rethrowMsg && se->is<MsgException>()) throw;
        return se;
    }
    catch (const exception& e) {
        return newExc<StdException>(mp, mp, loc, strsz(e.what()), current_exception(), &typeid(e));
    }
    catch (...) {
        return newExc<ExternalException>(mp, mp, loc, "Unknown exception", current_exception());
    }
}

}  // namespace ders
