/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#include <ders/fd_file.hpp>

namespace ders {

using namespace std;

fd_file::fd_file(mem_pool* mp, strsz path, int flags) : path_(mp, path), own_(true)
{
    error_code ec;
    fd_=impl::fd_open(path_.c_str(), flags, &ec);
    if (fd_<0) throw newExc<FileException>(mp, mp, FLLN, "Can't open", ec, path);
}

int fd_file::ex_read(void* buf, int n)
{
    auto mp=pool();
    error_code ec;

    int rc=read(buf, n, &ec);
    if (rc<0) throw newExc<FileException>(mp, mp, FLLN, "Can't read", ec, path());

    return rc;
}

int fd_file::ex_write(const void* buf, int n)
{
    auto mp=pool();
    error_code ec;

    int rc=write(buf, n, &ec);
    if (rc<0) throw newExc<FileException>(mp, mp, FLLN, "Can't write", ec, path());

    return rc;
}

long long fd_file::ex_tell()
{
    auto mp=pool();
    error_code ec;

    long long rc=tell(&ec);
    if (rc<0) throw newExc<FileException>(mp, mp, FLLN, "Can't tell", ec, path());

    return rc;
}

long long fd_file::ex_seek(long long off, whence whn)
{
    auto mp=pool();
    error_code ec;

    long long rc=seek(off, whn, &ec);
    if (rc<0) throw newExc<FileException>(mp, mp, FLLN, "Can't seek", ec, path());

    return rc;
}

}  // namespace ders
