/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#include <ders/dir.hpp>
#include <dirent.h>
#include <sys/stat.h>

namespace {

struct dir_guard
{
    DIR* d_;
    dir_guard(DIR* d) : d_(d) {}
    ~dir_guard() { closedir(d_); }
};
    
}  // unnamed

namespace ders {

using namespace std;    

vector<sh_text> list_dir(mem_pool* mp, strsz path, error_code* ec)
{
    ec->clear();

    DIR* d=opendir(text(mp, path).c_str());
    if (!d) {
        ec->assign(errno, system_category());
        return {};
    }
    dir_guard guard(d);

    vector<sh_text> ret;
    ret.reserve(100);

    for (dirent *e; (e=readdir(d)); ) {
        strsz name((char*)e->d_name);  // d_name is char[] so strsz(const char (&chn)[N]) doesn't work correctly
        ret.push_back(shtx(mp, name));
    }

    if (errno) ec->assign(errno, system_category());
    return ret;
}

file_info get_info(mem_pool* mp, strsz path, error_code* ec)
{
    ec->clear();
    file_info ret;

    struct stat statbuf;
    if (stat(text(mp, path).c_str(), &statbuf)==-1) {
        ec->assign(errno, system_category());
        return ret;
    }

    static_assert(sizeof(statbuf.st_size)>4, "long long required");
    ret.size=statbuf.st_size;

    if (S_ISREG(statbuf.st_mode)) ret.type=ft_file;
    else if (S_ISDIR(statbuf.st_mode)) ret.type=ft_dir;
    else ret.type=ft_other;

    return ret;
}

}  // namespace ders
