/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#include <ders/rdwr.hpp>

namespace ders {

bool buf_rd::get_line(text* line, bool noendl)    
{
    line->clear();

    for (int ch; ; ) {
        if ((ch=get())==-1) return line->size()>0;

        if (ch!='\n') *line+=ch;
        else {
            if (!noendl) *line+=ch;
            else {
                int sz=line->size();
                if (sz>0 && line->back()=='\r') line->uninitialized_resize(sz-1);
            }

            break;
        }
    }

    return true;
}

int buf_rd::read(void* buf, int n)    
{
    assert(n>=0);
    auto b1=static_cast<char*>(buf);
    auto b2=b1;

    while (n>0) {
        if (cur_==end_) {
            int rc=read_next();
            if (rc<=0) return (b2>b1) ? b2-b1 : rc;
        }

        int sz=end_-cur_;
        if (sz>n) sz=n;

        memcpy(b2, beg()+cur_, sz);

        b2+=sz;
        n-=sz;

        cur_+=sz;
    }

    return b2-b1;
}

int buf_rd::read(text* t, int n)
{
    assert(n>=0);

    t->clear();
    t->reserve(n);

    int rc=read(t->begin(), n);
    assert(rc<=n);

    if (rc!=-1) t->uninitialized_resize(rc);

    return rc;
}

int buf_rd::read_next()
{
    assert(cur_==end_);

    int rc=rd_->read(mb_.buf(), mb_.size(), &ec_);
    if (rc>0) {
        cur_=0;
        end_=rc;
    }

    return rc;
}

int buf_wr::write(const void* buf, int n)
{
    assert(n>=0);
    auto b1=static_cast<const char*>(buf);
    auto b2=b1;

    while (n>0) {
        if (cur_==mb_.size()) {
            int rc=write_next();
            if (rc<=0) return (b2>b1) ? b2-b1 : rc;
        }

        int sz=mb_.size()-cur_;
        if (sz>n) sz=n;

        memcpy(beg()+cur_, b2, sz);

        b2+=sz;
        n-=sz;

        cur_+=sz;
    }

    return b2-b1;
}

int buf_wr::write_next()
{
    assert(cur_>=0 && cur_<=mb_.size());
    if (cur_==0) return 0;

    int rc=wr_->write(mb_.buf(), cur_, &ec_);
    if (rc>0) {
        if (rc<cur_) memmove(beg(), beg()+rc, cur_-rc);
        cur_-=rc;
    }

    return rc;
}

}  // namespace ders
