/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#include <ders/text.hpp>

namespace ders {

const char* strsz::find(strsz s) const
{
    if (s.sz_>sz_) return 0;
    if (s.sz_==0) return str_;

    for (auto pos=str_; ; ) {
        int msz=sz_-(pos-str_)-(s.sz_-1);
        if (msz<=0) return 0;

        auto beg=(const char*)memchr(pos, s.str_[0], msz);
        if (!beg) return 0;

        if (beg[s.sz_-1]==s.str_[s.sz_-1]) {
            if (s.sz_<=2 || memcmp(beg+1, s.str_+1, s.sz_-2)==0) return beg;
        }

        pos=beg+1;
    }

    return 0;
}    

text& text::operator=(strsz s)
{
    int sz=s.size();
    reserve(sz);

    memcpy(buf_, s.str(), sz);
    used_=sz;

    return *this;
}

text& text::operator+=(strsz s)
{
    int sz=s.size(), newsz=used_+sz;
    reserve(newsz);

    memcpy(buf_+used_, s.str(), sz);
    used_=newsz;

    return *this;
}

void text::init(mem_pool* mp, strsz s, int real)
{
    assert(real>=1);
    mp_=mp;
    used_=s.size();
    resd_=(real>used_+1) ? real : used_+1;

    buf_=static_cast<char*>(mp_->alloc(resd_));
    memcpy(buf_, s.str(), used_);
}

void text::reserve_impl(int real)
{
    assert(real>resd_);
    if (resd_*2>real) real=resd_*2;

    char* newbuf=static_cast<char*>(mp_->alloc(real));
    memcpy(newbuf, buf_, used_);

    mp_->free(buf_, resd_);
    buf_=newbuf;
    resd_=real;
}

}  // namespace ders
