/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#include <ders/fd_file.hpp>
#include <vector>

using namespace std;
using namespace ders;

class MyException : public Exception {
 public:
    MyException(mem_pool* mp, FileLine loc, strsz msg) : Exception(mp, loc, msg) {}

    strsz className() const override { return strsz("MyException"); }
    void delete_this(mem_pool* mp) const override { delete_ptr(mp, this); }
};

void g(mem_pool *mp, int i)
{
    try {
        switch (i) {
        case 1: 
            throw newExc<MyException>(mp, mp, FLLN, "Hello from g()");
        case 2: {
            vector<int> v;
            v.at(0);
        }
        case 3: 
            throw 3;
        case 4: 
            throw newExc<ExitMsgException>(mp, mp, FLLN, "Goodbye from g()", i);
        }
    }
    catch (...) { throw newExc<Exception>(mp, FLLN, "Problems in g()", recatch(mp, FLLN)); }
}

void f(mem_pool *mp, int i)
{
    try { g(mp, i); }
    catch (...) { throw newExc<Exception>(mp, FLLN, "Problems in f()", recatch(mp, FLLN)); }
}

int main()
{
    mem_pool mpo, *mp=&mpo;
    fd_file out(mp, fd_out);

    for (int i=1; i<=4; i++) {
        try { f(mp, i); }
        catch (...) { 
            auto se=recatch(mp, FLLN, false);
            out.ex_write(tx_buf(mp)+"\tException #"+i+":\n"+toTextAll(get(se))+"\n");

            if (auto em=se->is<ExitMsgException>()) {
                out.ex_write(tx_buf(mp)+"\t"+em->message()+"\n");
                exit(em->exitCode());
            }
        }
    }
}
