/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#pragma once
#include <vector>
#include <ders/text.hpp>

namespace inchk {

using namespace ders;

struct IncFilePath {
    sh_text dirPath;
    sh_text incPath;
    sh_text fullPath;

    IncFilePath(sh_text dirp, sh_text incp);

    un_text to_text() const;
};

struct IncDirective {
    int line;
    sh_text qtdName;
    sh_text fullPath;

    IncDirective(int ln, sh_text qn, sh_text fp) : line(ln), qtdName(qn), fullPath(fp) {}

    un_text to_text() const;
};

struct IncFileData {
    std::vector<IncDirective> dirvs;
};

class IncFileDB : public deletable {
 public:
    virtual const IncFilePath* findByIncName(strsz in) const=0;
    virtual const IncFilePath* findByFullPath(strsz fp) const=0;
    virtual const IncFileData* getData(strsz path)=0;

    IncFileDB& operator=(const IncFileDB&)=delete;
};

un_ptr<IncFileDB> newIncFileDB(mem_pool* mp, const std::vector<sh_text>* incDirs);

struct IncLine {
    int line;
    sh_text name;
    bool quot;

    IncLine(int ln, sh_text nm, bool qt) : line(ln), name(nm), quot(qt) {}

    un_text to_text() const;
};

std::vector<IncLine> readIncLines(mem_pool* mp, strsz path);
un_text resolveIncLine(mem_pool* mp, strsz path, const IncLine* il, const IncFileDB* db);

}  // namespace inchk
