/*
 * Copyright (c) Sergey P. Derevyago, 2023.
 *
 * Distributed under the Boost Software License, Version 1.0.
 * http://www.boost.org/LICENSE_1_0.txt
 *
 */

#include "print.hpp"
#include <algorithm>
#include <ders/hash.hpp>

using namespace std;
using namespace ders;
using namespace inchk;

namespace {

const int LEVSZ=4;

using USet=mp_unordered_set<sh_text>;
using Alloc=USet::allocator_type;

vector<sh_text> print_impl(int lev, fd_file* out, strsz path, IncFileDB* db, USet* allInc)
{
    auto mp=out->pool();
    if (!allInc->insert(shtx(mp, path)).second) return {};

    text indent(mp, LEVSZ*10);
    while (indent.size()<LEVSZ*lev) indent+=' ';

    if (lev==0) out->ex_write(tx_buf(mp)+indent+path+":\n");

    vector<sh_text> included;

    auto data=db->getData(path);
    for (auto it=data->dirvs.begin(), end=data->dirvs.end(); it!=end; ++it) {
        tx_buf tb(mp);

        tb+indent+it->line+": "+it->qtdName;
        if (it->fullPath->size()) tb+" -> "+it->fullPath;
        tb+"\n";

        out->ex_write(tb);

        auto fname=it->fullPath;
        if (!fname->size()) {
            auto s=it->qtdName->str();
            fname=shtx(mp, strsz(s.str()+1, s.size()-2));
        }

        if (find(included.begin(), included.end(), fname)!=included.end()) {
            out->ex_write(tx_buf(mp)+"warning: "+path+":"+it->line+": "+it->qtdName+" already included\n");
        }
        included.push_back(fname);
//out->ex_write(tx_buf(mp)+"("+fname+")\n");

        if (it->fullPath->size()) {
            auto nested=print_impl(lev+1, out, it->fullPath->str(), db, allInc);
            included.insert(included.end(), nested.begin(), nested.end());
        }
    }

    if (lev==0) out->ex_write("\n");

    return included;
}

}  // unnamed

namespace inchk {

void print(fd_file* out, strsz path, IncFileDB* db)
{
    auto mp=out->pool();

    try {
        USet allInc{Alloc(mp)};
        print_impl(0, out, path, db, &allInc);
    }
    catch (...) { throw newExc<Exception>(mp, FLLN, "Can't print source file", recatch(mp, FLLN)); }
}

}  // namespace inchk
